# Comfy.JS
![npm](https://img.shields.io/npm/v/comfy.js?style=flat-square) ![GitHub](https://img.shields.io/github/license/instafluff/comfyjs?style=flat-square) [![](https://data.jsdelivr.com/v1/package/npm/comfy.js/badge)](https://www.jsdelivr.com/package/npm/comfy.js)

We built this Comfy Twitch Chat Module live on Twitch for Coding Cafe!

**Special Thanks:** *Comfy.JS is possible thanks to [tmi.js](https://tmijs.com/) maintained by [@AlcaDesign](https://github.com/AlcaDesign)*

**Comfy.JS** lets you integrate with Twitch chat for your Twitch channel ***SUPER EASILY*** in just a few lines of code. Here's a quick 3-min video on how to use it: (Click image to open video)

[![ComfyJS How-To Video](https://img.youtube.com/vi/oXpPwnUQCCk/hqdefault.jpg)](https://www.youtube.com/watch?v=oXpPwnUQCCk)

## Instafluff ##
> *Like these projects? The best way to support my open-source projects is by becoming a Comfy Sponsor on GitHub!*

> https://github.com/sponsors/instafluff

> *Come and hang out with us at the Comfiest Corner on Twitch!*

> https://twitch.tv/instafluff

## Instructions ##

#### Node
1. Install `comfy.js`
```
npm install comfy.js --save
```

2. Respond to !commands your channel
```javascript
var ComfyJS = require("comfy.js");
ComfyJS.onCommand = ( user, command, message, flags, extra ) => {
  if( flags.broadcaster && command === "test" ) {
    console.log( "!test was typed in chat" );
  }
}
ComfyJS.Init( "MyTwitchChannel" );
```

#### Browser
1. Download and add `comfy.js` from the `dist` folder or include from the JSDelivr CDN:
```javascript
<script src="comfy.min.js"></script>
```
OR
```javascript
<script src="https://cdn.jsdelivr.net/npm/comfy.js@latest/dist/comfy.min.js"></script>
```

2. Respond to !commands your channel
```html
<html>
  <head>
    <script src="https://cdn.jsdelivr.net/npm/comfy.js@latest/dist/comfy.min.js"></script>
  </head>
  <body>
    <script type="text/javascript">
      ComfyJS.onCommand = ( user, command, message, flags, extra ) => {
        if( flags.broadcaster && command === "test" ) {
          console.log( "!test was typed in chat" );
        }
      }
      ComfyJS.Init( "MyTwitchChannel" );
    </script>
  </body>
</html>
```

## Flags ##

Currently, the flags possible in `onCommand()` and `onChat()` are:

- broadcaster
- mod
- founder
- subscriber
- vip
- highlighted
- customReward

## Extra Parameter ##

Currently, the `extra` parameter for the `onCommand()` contains the following fields:

- id (the message message)
- channel
- roomId
- messageType
- messageEmotes
- isEmoteOnly
- userId
- username
- displayName
- userColor
- userBadges
- flags
- timestamp
- customRewardId (only works with custom channel rewards with required-text)

If the message is a command, the `extra` parameter will contain an additional field:

- sinceLastCommand

which contains the information on the time periods in `ms` since the last time any user, or the specific user, has used the same
command. This field can be convenient to be used for setting global cooldown or spamming filters. See examples below:

```javascript
ComfyJS.onChat = ( user, message, flags, self, extra ) => {
  if( flags.broadcaster && command === "test" ) {
    if( extra.sinceLastCommand.any < 100 ) {
      console.log(
        `The last '!test' command by any user was sent less than 100 ms ago`
      );            
    }

    if( extra.sinceLastCommand.user < 100 ) {
      console.log(
        `The last '!test' command by this specific user (as denoted by the 'user' parameter) was sent less than 100 ms ago`
      );            
    }
  }
}
```

## Reading Chat Messages ##

You can read chat messages by using the `onChat()` handler

```javascript
ComfyJS.onChat = ( user, message, flags, self, extra ) => {
  console.log( user, message );
}
```

## Sending Chat Messages ##

Sending Chat Messages can be done through `ComfyJS.Say( message )` but requires an OAUTH password when connecting to chat.

#### Securely adding your password
1. Get a Twitch Chat OAuth Password Token - [http://twitchapps.com/tmi/](http://twitchapps.com/tmi/)
2. Install `dotenv`
```
npm install dotenv --save
```
3. Create a file named `.env` that looks like this:
```javascript
TWITCHUSER=[YOUR-USERNAME-HERE]
OAUTH=[YOUR-OAUTH-PASS HERE] # e.g. OAUTH=oauth:kjh12bn1hsj78445234
```
4. Initialize with the Username and OAUTH password
```javascript
var ComfyJS = require("comfy.js");
ComfyJS.onCommand = ( user, command, message, flags, extra ) => {
  if( command === "test" ) {
    ComfyJS.Say( "replying to !test" );
  }
}
ComfyJS.Init( process.env.TWITCHUSER, process.env.OAUTH );
```

## Joining a Different Channel ##

You can join a different channel or groups of channels by specifying in the `Init()`

#### Joining a Single Channel

```javascript
ComfyJS.Init( "MyTwitchChannel", null, "ChannelToJoin" );
```

#### Joining Multiple Channels

```javascript
ComfyJS.Init( "MyTwitchChannel", null, [ "ChannelA", "ChannelB", "ChannelC" ] );
```

## Channel Point Reward Redemptions

Channel Point Reward Redemptions require extra Twitch OAuth permission scopes (and ***must be the OAuth of the channel owner***!)

You can use this tool: [https://twitchapps.com/tokengen/](https://twitchapps.com/tokengen/)

Scopes: `channel:manage:redemptions channel:read:redemptions user:read:email chat:edit chat:read`

Instafluff's Scopes List: `user:read:email user:read:chat chat:edit chat:read user:read:whispers user:manage:whispers channel:read:redemptions channel:read:hype_train channel:read:ads channel:read:charity channel:read:goals channel:read:guest_star channel:read:polls channel:read:predictions moderator:read:chatters moderator:read:followers user:write:chat channel:manage:redemptions moderator:manage:shoutouts channel:manage:polls channel:manage:predictions channel:manage:broadcast channel:manage:raids moderator:manage:announcements moderator:manage:automod moderator:manage:banned_users moderator:manage:chat_messages clips:edit`

```javascript
ComfyJS.onReward = ( user, reward, cost, message, extra ) => {
  console.log( user + " redeemed " + reward + " for " + cost );
}
```

Comfy.JS includes functions to manage Channel Point Rewards. These functions require the ClientID used in getting the Twitch OAuth password for the channel.
 - **GetChannelRewards**`( clientId, manageableOnly = true )`
    - Gets the channel's rewards
 - **CreateChannelReward**`( clientId, rewardInfo )`
    - Creates a channel point reward
 - **UpdateChannelReward**`( clientId, rewardId, rewardInfo )`
    - Updates a channel point reward *(Only rewards created with this ClientID can be updated)*
 - **DeleteChannelReward**`( clientId, rewardId )`
    - Deletes a channel point reward *(Only rewards created with this ClientID can be deleted)*

```javascript
let channelRewards = await ComfyJS.GetChannelRewards( clientId, true );

let customReward = await ComfyJS.CreateChannelReward( clientId, {
    title: "Test Reward",
    prompt: "Test Description",
    cost: 100,
    is_enabled: true,
    background_color: "#00E5CB",
    is_user_input_required: false,
    is_max_per_stream_enabled: false,
    max_per_stream: 0,
    is_max_per_user_per_stream_enabled: false,
    max_per_user_per_stream: 0,
    is_global_cooldown_enabled: false,
    global_cooldown_seconds: 0,
    should_redemptions_skip_request_queue: true
} );

let updatedReward = await ComfyJS.UpdateChannelReward( clientId, customReward.id, {
    title: "Test Reward (Updated)",
    prompt: "Updated Description",
    cost: 200,
    is_enabled: true,
} );

await ComfyJS.DeleteChannelReward( clientId, customReward.id );
```

## Disconnecting from Server

You can disconnect from the server and all channels by using `Disconnect()`.

```javascript
ComfyJS.Disconnect();
```

## All Supported Events ##
 - **onCommand**`( user, command, message, flags, extra )`
    - Responds to "!" commands
 - **onChat**`( user, message, flags, self, extra )`
    - Responds to user chatting
 - **onWhisper**`( user, message, flags, self, extra )`
    - **REQUIRES EXTRA PERMISSION SCOPES**
    - Responds to user whisper event
    - Requires one of the following OAuth scopes: `user:read:whispers`, `user:manage:whispers`
 - **onMessageDeleted**`( id, extra )`
    - Responds to chat message deleted
 - **onReward**`( user, reward, cost, message, extra )`
    - **REQUIRES EXTRA PERMISSION SCOPES**
    - Responds to Channel Point Redemptions
 - **onJoin**`( user, self, extra )`
    - Responds to user joining the chat
 - **onPart**`( user, self, extra )`
    - Responds to user leaving the chat
 - **onHosted**`( user, viewers, autohost, extra )`
    - Responds to channel being hosted
    - Requires being authorized as the broadcaster
 - **onBan**`( bannedUsername, extra )`
    - Responds to a user being banned
 - **onTimeout**`( timedOutUsername, durationInSeconds, extra )`
    - Responds to a user being timed out
 - **onRaid**`( user, viewers, extra )`
    - Responds to raid event
 - **onCheer**`( user, message, bits, flags, extra )`
    - Responds to user cheering
 - **onSub**`( user, message, subTierInfo, extra )`
    - Responds to user channel subscription
 - **onResub**`( user, message, streamMonths, cumulativeMonths, subTierInfo, extra )`
    - Responds to user channel subscription anniversary
 - **onSubGift**`( gifterUser, streakMonths, recipientUser, senderCount, subTierInfo, extra )`
    - Responds to user gift subscription
 - **onSubMysteryGift**`( gifterUser, numbOfSubs, senderCount, subTierInfo, extra )`
    - Responds to user sending gift subscriptions
 - **onGiftSubContinue**`( user, sender, extra )`
    - Responds to user continuing gift subscription
 - **onHypeTrain**`( "begin" | "progress" | "end", level, progressToNextLevel, goalToNextLevel, totalHype, timeRemainingInMS, extra )`
    - **REQUIRES EXTRA PERMISSION SCOPES**
    - Responds to when a Hype Train is happening
    - Requires one of the following OAuth scopes: `channel:read:hype_train`
 - **onShoutout**`( channelDisplayName, viewerCount, timeRemainingInMS, extra )`
    - **REQUIRES EXTRA PERMISSION SCOPES**
    - Responds to when streamers shoutout another channel
    - Requires one of the following OAuth scopes: `moderator:read:shoutouts`, `moderator:manage:shoutouts`
 - **onPoll**`( "begin" | "progress" | "end" | "archive" | "close" | "delete", title, choices, votes, timeRemainingInMS, extra )`
    - **REQUIRES EXTRA PERMISSION SCOPES**
    - Responds to when a stream poll is happening
    - Requires one of the following OAuth scopes: `channel:read:polls`, `channel:manage:polls`
    - Choices and votes are sorted by # of votes automatically. Top voted choice(s) will be the first elements in `choices` and `votes`
 - **onPrediction**`( "begin" | "progress" | "lock" | "end" | "cancel", title, outcomes, topPredictors, timeRemainingInMS, extra )`
    - **REQUIRES EXTRA PERMISSION SCOPES**
    - Responds to when a stream prediction is happening
    - Requires one of the following OAuth scopes: `channel:read:predictions`, `channel:manage:predictions`
 - **onConnected**`( address, port, isFirstConnect )`
    - Responds when connecting to the Twitch chat.
 - **onReconnect**`( reconnectCount )`
    - Responds when attempting to reconnect to the Twitch chat.
 - **onError**`( error )`
    - Hook for Errors


## Credits ##
Thank you to all the participants of this project!

**Instafluff, Instafriend, Neo_TA, ChatTranslator, fydo, That_MS_Gamer, MrRayKoma, Amarogine, HunWalk, simrose4u, sparky_pugwash, soggycoffee, blackdawn1980, BooobieTrap, lizardqueen, TastyGamers101, MalForTheWin, SourBeers, Stay_Hydrated_Bot, codeaurora, DutchGamer46, TheHungerService, BungalowGlow, koralina_211, TominationTime, itsDeke, fd_god92, SushiDay, FlyToto_, Docpinecone, katori15, ScrtSolstice, QeraiX, superravemonster, Jwh1o1, Deitypotato, Stobie, Chlapicek99, tehWokes, SuperChihuahua, FranC312, FuriousFur, Moopaloo, CreativeBuilds, donaldwm, Zorchenhimer, Grognardian, ravavyr, Chibigirl24, DR4G0N_S4MUR41, PokemoHero, rekaj3773, cunavrito, TheGeekGeneration, DevMerlin, julieee22, malfunct, blazeninja3, pookiepew, xxMiabellexx, Rlchibi**

Thank you to everyone that helped in turning Comfy.JS into a browser module!

**Instafluff, Instafriend, ChatTranslator, Gilokk0, qbotv3, That_MS_Gamer, KitAnnLIVE, simrose4u, MacabreMan2, LuRiMer313, sparky_pugwash, AbbyFabby, sethorizer, julieee22, Numb3rY, Jwh1o1, baileydale, kevkab, Stay_Hydrated_Bot, DrJavaSaurus, stresstest, BungalowGlow, Dr_Zero, NiteCrawla, fd_god92, DrEriksen, codeheir, Talk2meGooseman, sneelps, cottonsmiles, DutchGamer46, LilyHazel, Kyoslilmonster, guthron, DragosNox, sciondragons, HonestDanGames, Xynal, MerlinLeWizard, FablesGames, BrainoidGames, donaldwm, Gharrotey, RIKACHET, HeyOhKei, DevMerlin, CrimsonKnightZero, ellie_pop, ItsNaomiArt, SomaPills, TheSabbyLife, bktdakid31, IsaisChannel, thegooseofwild, itsDeke, bubblesandunicorns, jellydance, MalForTheWin, Chibigirl24, Pearcington, RikuRinku, rockysenpai24, DEAD_P1XL, codeaurora, EndlessMoonfall, fromtheannex, Optik_Nerve, qerwtr546, REAZNxxx, GoonPontoon, JesseSkinner, roberttables, pookiepew, Lannonbr, SoG_Cuicui, Deitypotato, shalomhanukkahbneishimon, UpmostKek, xeiu, skatesubzero, kingswerv, K1ng440, kaisuke, kinbiko, malfunct, BooobieTrap, Kara_Kim**

Thanks to everyone who joined in on adding Twitch PubSub support for Channel Point Reward Redemptions to Comfy.JS!

**Instafluff, Instafriend, informathemusic, aRandomTim, shadesofpixie, That_MS_Gamer, ToeNeeHee, httpJunkie, ryanchetty_1, calguru, chrislocality, Atanerah, rekaj3773, moshiko777, fizhes, AnnaCodes, Smokestormx, TheGeekGeneration, SavenaV, KotaKlan, rosebutterfly24, Simpathey, Spationaute, DjDesidera, JupiterZky, judybelle1, Shaezonai, shineslove, airsickmammal, walaber, jellydance, LilyHazel, PainArtist, Nickloop_TTV, VerbatimStudios, silversurfer1989, BellaTriXrbsa, holloway87, Asherroth86, Tiwesday, not_your_point, JenDevelops, tenaciousw, Cuicui_off, stevis5, aranhawaii, DevMerlin, wabes1, jeckle, opti_21, sparky_pugwash, tommunist_64, DutchGamer46, DoctorArgus, simrose4u, DreamGardenPanda, onelineofme, stuyksoft, Simployed, JustinZedly, Rhedone, DrMikachu, Gurkenkater, MrDemonWolf, saltipretzelz, MerlinLeWizard, Kurokirisu, Juscekame, FuriousFur, andresurrego, MissNightcrawler, karatewump, DrillsKibo, florinpop17, Axell99Design, Ahmed_Riad_1, Keegan_GDiegen, PortaalGaming, mjewl, cheppy4444dude, Soccerdude4444, klforthwind, penguinian, 10TenArt, Atndesign, DNIStream, LoveSudoNimh, prosto_artem27, lucasnramos, A_Ninja_For_Jesus_Bruh, RedChrisMS, Lineyatronic, Totte292, A_Gold_Fish, ShiDotMoe, tbdgamer, MatthewDGroves, dota2attitude, mistersyntax, SekaCakes, llamakid29, CryptoCoyote, MurdocTurner, JeanValjean80, walpolea, Jessi8712, butschibuuuu, Cmiley6, TheFlamingWings, hehe24h, cryogen_sw, DrJavaSaurus, rota22_, julieee22, bronick16, ScrtSolstice, ghostlupo86, wake_the_beast, williamcameron2, GizmoPugLife, OG24com**
